//
// osm.cpp
//
// Version 1.0 
//
// Code for the Open Street Map (OSM) classes.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

#include "Poco/SAX/SAXParser.h"
#include "Poco/SAX/ContentHandler.h"
#include "Poco/SAX/Attributes.h"
#include "Poco/Exception.h"

#include <map>
#include <list>
#include <ostream>
#include <iostream>

using Poco::XML::SAXParser;
//using Poco::XML::XMLReader;
//using Poco::XML::XMLString;
//using Poco::XML::XMLChar;
//using Poco::XML::ContentHandler;
//using Poco::XML::Attributes;
//using Poco::XML::Locator;
using namespace std;

#include "osm.h"
#include "osmhandler.h"

bool OSMNode::operator<(OSMNode other)
{
	return (NodeID < other.NodeID);
}

OSMNode::OSMNode()
{
	NodeID = 0;
	lat = 0.0;
	lon = 0.0;
}

OSMNode::OSMNode(int Id)
{
	NodeID = Id;
	lat = 0.0;
	lon = 0.0;
}

OSMNode::~OSMNode()
{
	tags.clear();
}


void OSMNode::AddTag(string ky, string vl)
{
	tags.insert(pair<string,string>(ky, vl));
}

void OSMNode::DisplayNode(ostream& s)
{
	multimap<string,string>::iterator TagsIt;
	
	s << "Node ID: " << NodeID << ", lat: " << lat << ", lon: " << lon << "\n";
	if (tags.size() > 0)
	{
		s << "Tags:\n";
		for (TagsIt = tags.begin(); TagsIt != tags.end(); TagsIt++)
		{
			s << "  " << (*TagsIt).first << "=" << (*TagsIt).second << "\n";  
		}
	}
	else
	{
		s << "No tags\n";
	}
	s << "\n";
}

OSMWay::OSMWay()
{
	WayID = 0;
}

OSMWay::OSMWay(int Id)
{
	WayID = Id;
}

OSMWay::~OSMWay()
{
	tags.clear();
	nodes.clear();
}

void OSMWay::AddNode(int nd)
{
	nodes.push_back(nd);
}

void OSMWay::AddTag(string ky, string vl)
{
	tags.insert(pair<string,string>(ky, vl));
}

void OSMWay::DisplayWay(ostream& s)
{
	multimap<string,string>::iterator TagsIt;
	list<int>::iterator NodesIt;
	
	s << "Way ID: " << WayID << "\n";
	if (nodes.size() > 0)
	{
		s << "Nodes:\n";
		for (NodesIt = nodes.begin(); NodesIt != nodes.end(); NodesIt++)
		{
			s << "  " << *NodesIt << "\n";  
		}
	}
	else
	{
		s << "*** Warning: No Nodes ***\n";
	}

	if (tags.size() > 0)
	{
		s << "Tags:\n";
		for (TagsIt = tags.begin(); TagsIt != tags.end(); TagsIt++)
		{
			s << "  " << (*TagsIt).first << "=" << (*TagsIt).second << "\n";  
		}
	}
	else
	{
		s << "No tags\n";
	}
	s << "\n";
}

OSMRelMember::OSMRelMember()
{
	type="";
	ref=0;
	role="";
}

OSMRelMember::OSMRelMember(string t, int rf, string rl)
{
	type=t;
	ref=rf;
	role=rl;
}

void OSMRelMember::DisplayMember(ostream& s)
{
	s << "Type:" << type << ", ref:" << ref << ", role:" << role << "\n";
}

OSMRelation::OSMRelation()
{
	RelID = 0;
}

OSMRelation::OSMRelation(int Id)
{
	RelID = Id;
}

OSMRelation::~OSMRelation()
{
	tags.clear();
	members.clear();
}

void OSMRelation::AddTag(string ky, string vl)
{
	tags.insert(pair<string,string>(ky, vl));
}

void OSMRelation::AddMember(OSMRelMember mem)
{
	members.push_back(mem);
}

void OSMRelation::DisplayRelation(ostream& s)
{
	multimap<string,string>::iterator TagsIt;
	list<OSMRelMember>::iterator MembersIt;
	
	s << "Relation ID: " << RelID << "\n";
	if (members.size() > 0)
	{
		s << "Members:\n";
		for (MembersIt = members.begin(); MembersIt != members.end(); MembersIt++)
		{
			s << "  ";
			(*MembersIt).DisplayMember(s);  
		}
	}
	else
	{
		s << "*** Warning: No Members ***\n";
	}

	if (tags.size() > 0)
	{
		s << "Tags:\n";
		for (TagsIt = tags.begin(); TagsIt != tags.end(); TagsIt++)
		{
			s << "  " << (*TagsIt).first << "=" << (*TagsIt).second << "\n";  
		}
	}
	else
	{
		s << "No tags\n";
	}
	
	s << "\n";
}

