//
// osmdata.h
//
// Version 1.0
//
// Source for the Open Street Map (OSM) data lists.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

#include "Poco/SAX/SAXParser.h"
#include "Poco/SAX/ContentHandler.h"
#include "Poco/SAX/Attributes.h"
#include "Poco/Exception.h"

#include <map>
#include <list>
#include <ostream>
#include <string>
#include <iostream>

#include "osmdata.h"
#include "osmhandler.h"
#include "osm.h"

OSMData::OSMData()
{
}

OSMData::~OSMData()
{
	NodeList.clear();
	WayList.clear();
	RelList.clear();
}

void OSMData::LoadFromFile(std::string fname)
{
	//OSMHandler handler(NodeList);
	OSMHandler handler;
	
	std::cout << "Loading from " << fname << " ..." << std::endl;
	
	
	Poco::XML::SAXParser parser;
	parser.setFeature(Poco::XML::XMLReader::FEATURE_NAMESPACES,true);
	parser.setFeature(Poco::XML::XMLReader::FEATURE_NAMESPACE_PREFIXES, true);
	parser.setContentHandler(&handler);
	
	try
	{
		parser.parse(fname);
	}
	catch (Poco::Exception& e)
	{
		std::cerr << e.displayText() << std::endl;
	}
	
	std::cout << "Nodes " << handler.NodeList.size() << std::endl;
	NodeList = handler.NodeList;
	WayList = handler.WayList;
	RelList = handler.RelList;
	handler.NodeList.clear();
	handler.WayList.clear();
	handler.RelList.clear();
}

