//
// osmdata.h
//
// Version 1.0 
//
// Header for the Open Street Map (OSM) data lists.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

#ifndef OSM_OSMDATA_H
#define OSM_OSMDATA_H

#include "osm.h"

class OSMData
//
// A class to contain the data loaded from a single download. A download could 
// contain any or all of nodes, ways and relations.  Each node is added to 
// NodeList, each way to WayList and each relation to RelList. Every node 
// referred to in a way should be in the NodeList. The nodes, ways and relations
// referred to in a relation might *NOT* be in the NodeList, WayList or RelList.
{
public:
	std::list<OSMNode> NodeList;
		// List of nodes.
	std::list<OSMWay> WayList;
		// List of ways
	std::list<OSMRelation> RelList;
		// List of relations
	
	OSMData();
		// Constructor for the class
	~OSMData();
		// Destructor for the class.  The lists are each destroyed.
	
	void LoadFromFile(std::string);
};

#endif
