//
// osmhandler.cpp
//
// Version 1.0
//
// Source for the Open Street Map (OSM) XML Sax handler.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//
#include "Poco/SAX/SAXParser.h"
#include "Poco/SAX/ContentHandler.h"
#include "Poco/SAX/Attributes.h"
#include "Poco/Exception.h"
#include <list>
#include <map>
#include <string>
#include <iostream>

#include "osm.h"
#include "osmhandler.h"

using Poco::XML::SAXParser;
using Poco::XML::XMLReader;
using Poco::XML::XMLString;
using Poco::XML::XMLChar;
using Poco::XML::ContentHandler;
using Poco::XML::Attributes;
using Poco::XML::Locator;
using namespace std;


OSMHandler::OSMHandler(): _pLocator(0)
{
}
	
void OSMHandler::setDocumentLocator(const Locator* loc)
{
	_pLocator = loc;
}
	
void OSMHandler::startDocument()
{
	where("startDocument");
}
void OSMHandler::endDocument()
{
	where("endDocument");
}
void OSMHandler::startElement(const XMLString& uri, const XMLString& localName, const XMLString& qname, const Attributes& attributes)
{
	if (qname=="osm")
	{
	}
	else if (qname=="node")
	{
		CurrElem=qname;
		nd.NodeID = atoi(attributes.getValue("id").c_str());
		nd.lon = atof(attributes.getValue("lon").c_str());
		nd.lat = atof(attributes.getValue("lat").c_str());
	}
	else if (qname=="way")
	{	
		CurrElem=qname;
		wy.WayID = atoi((attributes.getValue("id")).c_str());
	}
	else if (qname=="relation")
	{
		CurrElem=qname;
		rl.RelID = atoi((attributes.getValue("id")).c_str());
	}
	else if (qname=="tag")
	{
		if (CurrElem=="node")
		{
			nd.AddTag(attributes.getValue("k"),attributes.getValue("v"));
		}
		else if (CurrElem=="way")
		{
			wy.AddTag(attributes.getValue("k"),attributes.getValue("v"));
		}
		else if (CurrElem=="relation")
		{
			rl.AddTag(attributes.getValue("k"),attributes.getValue("v"));
		}
	}
	else if (qname=="nd")
	{
		if (CurrElem=="way")
		{
			wy.AddNode(atoi(attributes.getValue("ref").c_str()));
		}
		else
		{
			cout << "nd attached to " << CurrElem << endl; 
		}
	}
	else if (qname=="member")
	{
		if (CurrElem=="relation")
		{
			mb.type = attributes.getValue("type");
			mb.ref = atoi(attributes.getValue("ref").c_str());
			mb.role = attributes.getValue("role");
			rl.AddMember(mb);
		}
	}
	else
	{
			std::cout << "Unknown element: " << qname << std::endl;
	}
}
void OSMHandler::endElement(const XMLString& uri, const XMLString& localName, const XMLString& qname)
{
	if (CurrElem=="node" && qname=="node")
	{
		NodeList.push_back(nd);
		nd.tags.clear();
		CurrElem="none";
	}
	else if (CurrElem=="way" && qname=="way")
	{
		WayList.push_back(wy);
		wy.tags.clear();
		wy.nodes.clear();
		CurrElem="none";
	}
	else if (CurrElem=="relation" && qname=="relation")
	{
		RelList.push_back(rl);
		rl.tags.clear();
		rl.members.clear();
		CurrElem="none";
	}
	else if (qname == "tag" || qname == "nd" || qname == "member")
	{
	}
	else
	{
		cout << "Unknown end. CurrElem=" << CurrElem << " qname=" << qname << endl;
	}
}
void OSMHandler::characters(const XMLChar ch[], int start, int length)
{

}
void OSMHandler::ignorableWhitespace(const XMLChar ch[], int start, int length)
{
	where("ignorableWhitespace");
}
void OSMHandler::processingInstruction(const XMLString& target, const XMLString& data)
{
	where("processingInstruction");
	std::cout << "target=" << target << " , data=" << data << std::endl;
}
void OSMHandler::startPrefixMapping(const XMLString& prefix, const XMLString& uri)
{
	where("startPrefixMapping");
	std::cout << "prefix=" << prefix << " uri=" << uri << std::endl;
}
void OSMHandler::endPrefixMapping(const XMLString& prefix)
{
	where("endPrefixMapping");
	std::cout << "prefix=" << prefix << std::endl;
}
void OSMHandler::skippedEntity(const XMLString& name)
{
	where("skippedEntity");
	std::cout << "name=" << name << std::endl;
}
	
void OSMHandler::comment(const XMLChar ch[], int start, int length)
{
	where("comment");
}
void OSMHandler::where(const std::string& meth)
{
	std::cout << "*** " << meth;
	if (_pLocator)
	{
		std::cout << " in "
						  << _pLocator->getSystemId()
						  << ", line " << _pLocator->getLineNumber()
						  << ", col " << _pLocator->getColumnNumber()
						  << std::endl;
	}
}
