//
// osmhandler.h
//
// Version 1.0 
//
// Header for the Open Street Map (OSM) XML Sax handler.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

#ifndef OSM_OSMHANDLER_H
#define OSM_OSMHANDLER_H

class OSMHandler: public Poco::XML::ContentHandler
// A Sax handler the parser hands each element from the XML file to the relevant
// function below.  Functions that extract and process data store it in lists
// in this class, which can them be copied and cleaned.
{
public:
	std::list<OSMNode> NodeList;
		// List of OSM nodes.
	std::list<OSMWay> WayList;
		// List of OSM ways
	std::list<OSMRelation> RelList;
		// List of OSM relations
	
	OSMHandler();
	
	void setDocumentLocator(const Poco::XML::Locator* loc);
	
	void startDocument();
	void endDocument();
	void startElement(const Poco::XML::XMLString& uri, const Poco::XML::XMLString& localName, const Poco::XML::XMLString& qname, const Poco::XML::Attributes& attributes);
	void endElement(const Poco::XML::XMLString& uri, const Poco::XML::XMLString& localName, const Poco::XML::XMLString& qname);
	void characters(const Poco::XML::XMLChar ch[], int start, int length);
	void ignorableWhitespace(const Poco::XML::XMLChar ch[], int start, int length);
	void processingInstruction(const Poco::XML::XMLString& target, const Poco::XML::XMLString& data);
	void startPrefixMapping(const Poco::XML::XMLString& prefix, const Poco::XML::XMLString& uri);
	void endPrefixMapping(const Poco::XML::XMLString& prefix);
	void skippedEntity(const Poco::XML::XMLString& name);	
	void comment(const Poco::XML::XMLChar ch[], int start, int length);
protected:
	void where(const std::string& meth);
	
private:
	const Poco::XML::Locator* _pLocator; 
	
	OSMNode nd;
	OSMWay wy;
	OSMRelation rl;
	OSMRelMember mb;
	
	std::string CurrElem;
};

#endif
